pub struct PoisonGuard<'a, G> { /* private fields */ }Expand description
An RAII guard for a Poisonable. When this structure is dropped (falls
out of scope), the lock will be unlocked.
The data protected by the underlying lock can be accessed through this
guard via its Deref and DerefMut implementations.
This method is created by calling the lock, try_lock, read, and
try_read methods on Poisonable
This guard holds a ThreadKey, so it is not possible to lock anything
else until this guard is dropped. The ThreadKey can be reacquired by
calling Poisonable::unlock, or Poisonable::unlock_read.